@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: comedi.c,v 1.8 2005/06/28 21:28:39 baccala Exp $
+ * @(#)$Id: comedi.c,v 2.4 2009/01/17 06:44:55 baccala Exp $
  *
  * Author: Brent Baccala <baccala@freesoft.org>
  *
@@ -41,6 +41,7 @@
  */
 
 comedi_t *comedi_dev = NULL;
+gchar *comedi_board_name = NULL;
 
 static int comedi_opened = 0;	/* t if open has at least been _attempted_ */
 static int comedi_running = 0;
@@ -74,6 +75,8 @@ static int bufvalid=0;
 // has to be set later
 int zero_value = -1;
 
+static int lag = 0;			/* lag - see get_data() */
+
 static int subdevice_flags = 0;
 static int subdevice_type = COMEDI_SUBD_UNUSED;
 
@@ -411,6 +414,8 @@ close_comedi()
 
   if (comedi_dev) comedi_close(comedi_dev);
   comedi_dev = NULL;
+  if (comedi_board_name) g_free(comedi_board_name);
+  comedi_board_name = NULL;
   comedi_running = 0;
   comedi_opened = 0;
 
@@ -452,7 +457,7 @@ open_comedi(void)
      */
 
     for (i = 0 ; i < NCHANS ; i++) {		/* XXX hardwired at 8 */
-      memset(comedi_chans[i].data, 0, MAXWID * sizeof(short));
+      comedi_chans[i].data = NULL;
       comedi_chans[i].num = comedi_chans[i].frame = comedi_chans[i].volts = 0;
       comedi_chans[i].listeners = 0;
       //sprintf(comedi_chans[i].name, "Channel %d", i);
@@ -471,6 +476,13 @@ open_comedi(void)
     return 0;
   }
 
+  /* All the GTK stuff uses UTF8, and complains to stderr if it
+   * doesn't get it.
+   */
+
+  comedi_board_name = g_locale_to_utf8(comedi_get_board_name(comedi_dev),
+				       -1, NULL, NULL, NULL);
+
   /* XXX I'd kinda like to do this here, but then the read() loop
    * below (to get offset correction for the DAQP) returns errors
    * (-EAGAIN).  If do this later, and start_comedi_running() has
@@ -479,7 +491,7 @@ open_comedi(void)
    */
   /* fcntl(comedi_fileno(comedi_dev), F_SETFL, O_NONBLOCK); */
 
-  if (strncmp(comedi_get_board_name(comedi_dev), "DAQP", 4) == 0) {
+  if (comedi_board_name && strncmp(comedi_board_name, "DAQP", 4) == 0) {
 
     /* Special case for DAQP - unfortunately, COMEDI doesn't (yet) provide
      * a generic interface for boards that can do offset correction,
@@ -490,7 +502,7 @@ open_comedi(void)
      */
 
     comedi_cmd cmd;
-    int chan;
+    unsigned int chan;
     int ret;
 
     ret = comedi_get_cmd_generic_timed(comedi_dev, comedi_subdevice, &cmd, 0);
@@ -699,6 +711,8 @@ static void set_width(int width)
 
   for (i=0; i<NCHANS; i++) {
     comedi_chans[i].width = width;
+    if (comedi_chans[i].data != NULL) free(comedi_chans[i].data);
+    comedi_chans[i].data = malloc(width * sizeof(short));
   }
 }
 
@@ -855,6 +869,7 @@ static int get_data(void)
 	      memcpy(buf, (char *) buf + bytes_read - bufvalid, bufvalid);
 	    }
 
+	    lag = 0;
 	    return triggered;
 	  }
 	}
@@ -888,6 +903,11 @@ static int get_data(void)
      *
      * comedi-0.7.60 returned EINVAL on buffer overflows;
      * comedi-0.7.66 returns EPIPE
+     *
+     * In any event, if we detect an overflow, we reset the capture to
+     * start a new trace, record how many microseconds elapsed since
+     * the last time we were able to read the device, and report this
+     * on the screen as "lag".
      */
 
     if (errno != EINVAL && errno != EPIPE) perror("comedi read");
@@ -895,11 +915,11 @@ static int get_data(void)
     start_comedi_running();
     bufvalid = 0;
     gettimeofday(&tv2, NULL);
-    fprintf(stderr, "%ld microseconds lag\n",
-	    1000000*(tv2.tv_sec-tv1.tv_sec) + tv2.tv_usec - tv1.tv_usec);
+    lag = 1000000*(tv2.tv_sec-tv1.tv_sec) + tv2.tv_usec - tv1.tv_usec;
     return 0;
   }
 
+  lag = 0;
   return triggered;
 }
 
@@ -914,7 +934,7 @@ static char * status_str(int i)
     return comedi_devname;
   case 2:
     if (comedi_dev) {
-      return comedi_get_board_name(comedi_dev);
+      return comedi_board_name;
     } else {
       return split_field(error, 0, 16);
     }
@@ -936,6 +956,10 @@ static char * status_str(int i)
   case 3:
     if (comedi_dev && comedi_error) {
       return split_field(error, 0, 16);
+    } else if (lag > 1000) {
+      snprintf(buffer, sizeof(buffer), "%d ms lag", lag/1000);
+    } else if (lag > 0) {
+      snprintf(buffer, sizeof(buffer), "%d \302\265s lag", lag);
     } else {
       return "";
     }
@@ -994,7 +1018,13 @@ static char * option1str(void)
 }
 
 
-/* XXX Option 2 key - use this for a per-channel Range setting? */
+#if 0
+
+/* XXX Option 2 key - use this for a per-channel Range setting?
+ *
+ * If so, would also need to add option strings below.
+ */
+
 
 static int option2(void)
 {
@@ -1006,7 +1036,7 @@ static char * option2str(void)
   return NULL;
 }
 
-/* XXX add options for per-channel Range setting */
+#endif
 
 static int comedi_set_option(char *option)
 {
@@ -1104,8 +1134,13 @@ DataSrc datasrc_comedi = {
   status_str,
   option1,
   option1str,
+#if 0
   option2,
   option2str,
+#else
+  NULL,
+  NULL,
+#endif
   comedi_set_option,
   comedi_save_option,
   comedi_gtk_options,
