@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: display.h,v 1.12 2005/06/28 21:28:39 baccala Exp $
+ * @(#)$Id: display.h,v 2.4 2009/01/15 03:57:20 baccala Exp $
  *
  * Copyright (C) 1996 - 1999 Tim Witham <twitham@quiknet.com>
  *
@@ -21,24 +21,17 @@
 
 extern char fontname[];
 extern char fonts[];
-extern int color[];
 
-void	init_screen();		/* exported from display.c */
-void	init_widgets();
+void	init_widgets()		/* exported from display.c */;
 void	mainloop();
-void	draw_text();
+void	update_text();
+void	timebase_changed(void);
 void	clear();
-void	message();
+void	message(char *);
 void	cleanup_display();
 void	animate();
 int	col();
 
-void	DrawPixel();		/* these are defined in */
-void	DrawLine();		/* a display-specific file */
-void	SetColor();		/* like gr_vga.c or gr_sx.c */
-void	PolyPoint();
-void	PolyLine();
-void	SyncDisplay();
 void	AddTimeOut();
 void	LoadSaveFile();
 void	ExternCommand();
@@ -46,5 +39,3 @@ char *	GetFile();
 char *	GetString();
 int	GetYesNo();
 int	OpenDisplay();
-void	ClearDrawArea();
-void	MainLoop();
