@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: oscope.c,v 1.93 2004/11/04 19:47:33 baccala Exp $
+ * @(#)$Id: oscope.c,v 2.7 2009/01/15 03:57:20 baccala Exp $
  *
  * Copyright (C) 1996 - 2001 Tim Witham <twitham@quiknet.com>
  *
@@ -52,9 +52,6 @@ Channel ch[CHANNELS];
 char *progname;			/* the program's name, autoset via argv[0] */
 char version[] = VERSION;	/* version of the program, from Makefile */
 char error[256];		/* buffer for "one-line" error messages */
-int v_points;			/* pixels in vertical axis */
-int h_points;			/* pixels in horizontal axis */
-int offset;			/* vertical pixel offset to zero line */
 int clip = 0;			/* whether we're maxed out or not */
 char *filename;			/* default file name */
 int frames = 0;			/* # of frames (full or partial) captured */
@@ -82,8 +79,6 @@ Startup Options  Description (defaults)               version %s\n\
 -s <scale>       time Scale: 1/2000-1000/1 where 1=1ms/div    (%d/1)\n\
 -t <trigger>     Trigger level[:type[:channel]]               (%s)\n\
 -l <cursors>     cursor Line positions: first[:second[:on?]]  (%s)\n\
--c <color>       graticule Color: 0-15                        (%d)\n\
--m <mode>        video Mode (size): 0,1,2,3                   (%d)\n\
 -f <font name>   the Font name as-in %s\n\
 -p <type>        Plot mode: 0/1=point, 2/3=line, 4/5=step     (%d)\n\
 -g <style>       Graticule: 0=none,  1=minor, 2=major         (%d)\n\
@@ -94,7 +89,6 @@ file             %s file to load to restore settings and memory\n\
 ",
 	  progname, version, CHANNELS, CHANNELS, DEF_A,
 	  DEF_S, DEF_T, DEF_L,
-	  DEF_C, scope.size,
 	  fonts,		/* the font method for the display */
 	  scope.mode,
 	  scope.grat, def[DEF_B], def[!DEF_B],
@@ -129,7 +123,6 @@ cleanup()
 void
 init_scope()
 {
-  scope.size = DEF_M;
   scope.mode = DEF_P;
   scope.scale = DEF_S;
   scope.div = 1;
@@ -138,7 +131,6 @@ init_scope()
   scope.grat = DEF_G;
   scope.behind = DEF_B;
   scope.run = 1;
-  scope.color = DEF_C;
   scope.select = DEF_A - 1;
   scope.verbose = DEF_V;
   scope.min_interval = 50;
@@ -165,12 +157,22 @@ init_channels()
   }
 }
 
-/* samples needed to draw a current display of RATE */
+/* samples needed to draw a current display of RATE
+ *
+ * scope.div / scope.scale gives us the timebase in milliseconds per
+ * divisions.  Multiply by 10 to get millseconds per screen, divide by
+ * 1000 to get seconds per screen, multiply by rate (in samples/sec)
+ * to get samples per screen, and add one so rounding doesn't
+ * make us end a capture before the end of the screen.
+ */
+
 int
 samples(int rate)
 {
   static unsigned long int r;
-  r = (rate * ((h_points - 200) / 44) * scope.div / scope.scale / 1000) + 2;
+
+  r = rate * 10 * scope.div / scope.scale / 1000 + 1;
+
   if (r > MAXWID) r = MAXWID;
   return (r);
 }
@@ -391,16 +393,27 @@ void
 handle_key(unsigned char c)
 {
   static Channel *p;
-  static int s;
+  int displayed_samples, max_samples;
 
   p = &ch[scope.select];
-  s = p->signal ? samples(p->signal->rate) : 0;	/* needed for cursors */
+
+  /* These next two are used for keyboard movement of the cursors.
+   *
+   * Cursors are stored as sample numbers (1 based).  max_samples
+   * gives the largest legal value for a cursor.  displayed_samples
+   * gives the number of samples displayed on the screen at once so we
+   * can jump by 1/20 of a screen width.  If the signal is wider than
+   * the screen width (i.e, the scrollbar is on), then these two
+   * numbers will be different.
+   */
+
+  displayed_samples = p->signal ? samples(p->signal->rate) : 0;
+  max_samples = p->signal ? max(samples(p->signal->rate), p->signal->num) : 0;
 
   if (c >= 'A' && c <= 'Z') {
     if (p->signal) {
       save(c);			/* store channel */
       clear();			/* need this in case other chan displays mem */
-      draw_text(1);
     }
     return;
   } else if (c >= 'a' && c <= 'z') {
@@ -430,35 +443,35 @@ handle_key(unsigned char c)
     clear();
     break;
   case 'q' - 96:		/* -96 is CTRL keys */
-    if ((scope.cursa -= s / 20) < 1)
-      scope.cursa = s - 1;
+    if ((scope.cursa -= displayed_samples / 20) < 1)
+      scope.cursa = max_samples - 1;
     break;
   case 'w' - 96:
-    if ((scope.cursa += 2 / 20) >= s)
+    if ((scope.cursa += displayed_samples / 20) >= max_samples)
       scope.cursa = 1;
     break;
   case 'e' - 96:
     if (--scope.cursa < 1)
-      scope.cursa = s - 1;
+      scope.cursa = max_samples - 1;
     break;
   case 'r' - 96:
-    if (++scope.cursa >= s)
+    if (++scope.cursa >= max_samples)
       scope.cursa = 1;
     break;
   case 'a' - 96:
-    if ((scope.cursb -= s / 20) < 1)
-      scope.cursb = s - 1;
+    if ((scope.cursb -= displayed_samples / 20) < 1)
+      scope.cursb = max_samples - 1;
     break;
   case 's' - 96:
-    if ((scope.cursb += s / 20) >= s)
+    if ((scope.cursb += displayed_samples / 20) >= max_samples)
       scope.cursb = 1;
     break;
   case 'd' - 96:
     if (--scope.cursb < 1)
-      scope.cursb = s - 1;
+      scope.cursb = max_samples - 1;
     break;
   case 'f' - 96:
-    if (++scope.cursb >= s)
+    if (++scope.cursb >= max_samples)
       scope.cursb = 1;
     break;
   case '\t':
@@ -494,15 +507,15 @@ handle_key(unsigned char c)
     clear();
     break;
   case ']':
-    p->pos -= 16;		/* position up */
-    if (p->pos < -1 * v_points / 2)
-      p->pos = v_points / 2;
+    p->pos -= 0.1;		/* position up */
+    if (p->pos < -1) p->pos = -1;
+    if (p->pos * p->pos < 0.0001) p->pos = 0;
     clear();
     break;
   case '[':
-    p->pos += 16;		/* position down */
-    if (p->pos > v_points / 2)
-      p->pos = -1 * v_points / 2;
+    p->pos += 0.1;		/* position down */
+    if (p->pos > 1) p->pos = 1;
+    if (p->pos * p->pos < 0.0001) p->pos = 0;
     clear();
     break;
   case ';':
@@ -511,7 +524,7 @@ handle_key(unsigned char c)
       p->show = 1;
       clear();
     } else
-      message("Math can not run on Channel 1 or 2", KEY_FG);
+      message("Math can not run on Channel 1 or 2");
     break;
   case ':':
     if (scope.select > 1) {	/* previous math function */
@@ -519,21 +532,21 @@ handle_key(unsigned char c)
       p->show = 1;
       clear();
     } else
-      message("Math can not run on Channel 1 or 2", KEY_FG);
+      message("Math can not run on Channel 1 or 2");
     break;
   case '0':
     if (scope.div > 1)		/* decrease time scale, zoom in */
       scope.div = scaledown(scope.div);
     else
       scope.scale = scaleup(scope.scale, 5000);
-    clear();
+    timebase_changed();
     break;
   case '9':
     if (scope.scale > 1)	/* increase time scale, zoom out */
       scope.scale = scaledown(scope.scale);
     else
       scope.div = scaleup(scope.div, 2000);
-    clear();
+    timebase_changed();
     break;
   case '=':
     if (datasrc->set_trigger) {	/* increase trigger */
@@ -585,16 +598,6 @@ handle_key(unsigned char c)
       clear();
     }
     break;
-  case '<':
-    if (--scope.color < 0)	/* decrease color */
-      scope.color = 15;
-    draw_text(1);
-    break;
-  case '>':
-    if (++scope.color > 15)	/* increase color */
-      scope.color = 0;
-    draw_text(1);
-    break;
   case '@':			/* load file */
     LoadSaveFile(0);
     break;
@@ -605,7 +608,7 @@ handle_key(unsigned char c)
     if (scope.select > 1)
       ExternCommand();
     else
-      message("External commands can not run on Channel 1 or 2", KEY_FG);
+      message("Pipes can not run on Channel 1 or 2");
     break;
   case '&':			/* toggle between various data sources */
     if (datasrc_next()) {
@@ -636,7 +639,7 @@ handle_key(unsigned char c)
     break;
   case '.':
     scope.behind = !scope.behind; /* graticule behind/in front of signal */
-    draw_text(1);
+    update_text();
     break;
   case '?':
   case '/':
@@ -649,7 +652,7 @@ handle_key(unsigned char c)
       scope.run = 0;
     if ((scope.run == 1) && datasrc)
       setinputfd(datasrc->fd());	/* were stopped, so now start */
-    draw_text(1);
+    update_text();
     break;
   case '\r':
   case '\n':
@@ -685,8 +688,7 @@ main(int argc, char **argv)
     exit(1);
   parse_args(argc, argv);
   init_widgets();
-  init_screen();
-  /* clear(); */
+  clear();
 
   filename = FILENAME;
   if ((optind < argc) && (argv[optind] != NULL)) {
@@ -698,12 +700,12 @@ main(int argc, char **argv)
     exit(1);
 #endif
   }
-#if 0
-  init_widgets();
-  init_screen();
-#endif
+
   clear();
-  mainloop();
+  animate(NULL);
+
+  gtk_main();
+
   cleanup();
   exit(0);
 }
